@@ -18,9 +18,10 @@
 #include "semaphore.h"
 #include "internals.h"
 #include "restart.h"
+#include "spinlock.h"
 
 
-#ifndef HAS_COMPARE_AND_SWAP
+#if !defined HAS_COMPARE_AND_SWAP && !defined TEST_FOR_COMPARE_AND_SWAP
 /* If we have no atomic compare and swap, fake it using an extra spinlock.  */
 
 #include "spinlock.h"
@@ -34,6 +35,24 @@ static inline int compare_and_swap(sem_t *sem, long oldval, long newval)
   return ret;
 }
 
+#elif defined TEST_FOR_COMPARE_AND_SWAP
+
+static int has_compare_and_swap = -1; /* to be determined at run-time */
+
+static inline int compare_and_swap(sem_t *sem, long oldval, long newval)
+{
+  int ret;
+
+  if (has_compare_and_swap)
+    return __compare_and_swap(&sem->sem_status, oldval, newval);
+
+  acquire(&sem->sem_spinlock);
+  if ((ret = sem->sem_status == oldval) != 0)
+    sem->sem_status = newval;
+  release(&sem->sem_spinlock);
+  return ret;
+}
+
 #else
 /* But if we do have an atomic compare and swap, use it!  */
 
@@ -54,6 +73,7 @@ static inline int compare_and_swap(sem_t *sem, long oldval, long newval)
      2N+1, 2N-1, ..., 3, 1, &first_waiting_thread, &second_waiting_thread, ...
 */
 
+
 int sem_init(sem_t *sem, int pshared, unsigned int value)
 {
   if (value > SEM_VALUE_MAX) {
@@ -64,6 +84,14 @@ int sem_init(sem_t *sem, int pshared, unsigned int value)
     errno = ENOSYS;
     return -1;
   }
+#ifdef TEST_FOR_COMPARE_AND_SWAP
+  if (has_compare_and_swap == -1) {
+    has_compare_and_swap = compare_and_swap_is_available();
+  }
+#endif
+#if !defined HAS_COMPARE_AND_SWAP || defined TEST_FOR_COMPARE_AND_SWAP
+  sem->sem_spinlock = 0;
+#endif
   sem->sem_status = ((long)value << 1) + 1;
   return 0;
 }
@@ -71,8 +99,8 @@ int sem_init(sem_t *sem, int pshared, unsigned int value)
 int sem_wait(sem_t * sem)
 {
   long oldstatus, newstatus;
-  volatile pthread_t self = thread_self();
-  pthread_t * th;
+  volatile pthread_descr self = thread_self();
+  pthread_descr * th;
 
   while (1) {
     do {
@@ -81,7 +109,7 @@ int sem_wait(sem_t * sem)
         newstatus = oldstatus - 2;
       else {
         newstatus = (long) self;
-        self->p_nextwaiting = (pthread_t) oldstatus;
+        self->p_nextwaiting = (pthread_descr) oldstatus;
       }
     }
     while (! compare_and_swap(sem, oldstatus, newstatus));
@@ -105,8 +133,8 @@ int sem_wait(sem_t * sem)
          the net result is that at the time pthread_exit is called,
          self is no longer reachable from sem->sem_status. */
       if (oldstatus != (long) self && (oldstatus & 1) == 0) {
-        th = &(((pthread_t) oldstatus)->p_nextwaiting);
-        while (*th != (pthread_t) 1 && *th != NULL) {
+        th = &(((pthread_descr) oldstatus)->p_nextwaiting);
+        while (*th != (pthread_descr) 1 && *th != NULL) {
           if (*th == self) {
             *th = self->p_nextwaiting;
             break;
@@ -138,7 +166,7 @@ int sem_trywait(sem_t * sem)
 int sem_post(sem_t * sem)
 {
   long oldstatus, newstatus;
-  pthread_t th, next_th;
+  pthread_descr th, next_th;
 
   do {
     oldstatus = sem->sem_status;
@@ -155,13 +183,13 @@ int sem_post(sem_t * sem)
   }
   while (! compare_and_swap(sem, oldstatus, newstatus));
   if ((oldstatus & 1) == 0) {
-    th = (pthread_t) oldstatus;
+    th = (pthread_descr) oldstatus;
     do {
       next_th = th->p_nextwaiting;
       th->p_nextwaiting = NULL;
       restart(th);
       th = next_th;
-    } while(th != (pthread_t) 1);
+    } while(th != (pthread_descr) 1);
   }
   return 0;
 }
