@@ -1,5 +1,5 @@
 /* The weak pthread functions for Linux.
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -17,8 +17,12 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
+#include <errno.h>
 #include <stdlib.h>
 
+/* This is from <pthread.h>.  */
+typedef unsigned int pthread_key_t;
+
 extern int __pthread_return_0 __P ((void));
 extern int __pthread_return_1 __P ((void));
 extern void __pthread_return_void __P ((void));
@@ -63,7 +67,6 @@ weak_alias (__pthread_return_0, pthread_create)
 weak_alias (__pthread_return_0, pthread_join)
 weak_alias (__pthread_return_0, pthread_detach)
 weak_alias (__pthread_return_0, pthread_cond_timedwait)
-weak_alias (__pthread_return_0, pthread_key_create)
 weak_alias (__pthread_return_0, pthread_key_delete)
 weak_alias (__pthread_return_0, pthread_once)
 weak_alias (__pthread_return_0, pthread_cancel)
@@ -75,6 +78,9 @@ weak_alias (__pthread_return_void, pthread_testcancel)
 /* Those are pthread functions which return 1 if successful. */
 weak_alias (__pthread_return_1, pthread_equal)
 
+/* Those are pthread functions which return != 0 if unsuccessful. */
+weak_alias (__pthread_return_1, __pthread_key_create)
+
 /* pthread_exit () is a special case. */
 void
 weak_function
